@@ -18,6 +18,10 @@
 DLLEXPORT void
 domClearPSVIInList(xmlNodePtr list);
 
+static int _domIsDoc(xmlNodePtr node) {
+    return(node->type == XML_DOCUMENT_NODE || node->type == XML_HTML_DOCUMENT_NODE || node->type == XML_DOCB_DOCUMENT_NODE);
+}
+
 DLLEXPORT void
 domClearPSVI(xmlNodePtr tree) {
     xmlAttrPtr prop;
@@ -33,7 +37,7 @@ domClearPSVI(xmlNodePtr tree) {
             domClearPSVIInList(prop->children);
             prop = prop->next;
         }
-    } else if (tree->type == XML_DOCUMENT_NODE) {
+    } else if (_domIsDoc(tree)) {
         ((xmlDocPtr) tree)->psvi = NULL;
     }
     if (tree->children != NULL)
@@ -457,7 +461,7 @@ domIsParent( xmlNodePtr cur, xmlNodePtr refNode ) {
         return 0;
     }
 
-    if( refNode->type == XML_DOCUMENT_NODE ) {
+    if (_domIsDoc(refNode)) {
         return 1;
     }
 
@@ -507,7 +511,7 @@ domTestHierarchy(xmlNodePtr cur, xmlNodePtr refNode) {
 
 DLLEXPORT int
 domTestDocument(xmlNodePtr cur, xmlNodePtr refNode) {
-    if ( cur->type == XML_DOCUMENT_NODE || cur->type == XML_HTML_DOCUMENT_NODE) {
+    if ( _domIsDoc(cur) ) {
         switch ( refNode->type ) {
         case XML_ATTRIBUTE_NODE:
         case XML_ELEMENT_NODE:
@@ -549,9 +553,7 @@ domNodeIsReferenced(xmlNodePtr self) {
             }
         }
     }
-    else if (self->type == XML_DOCUMENT_NODE
-             || self->type == XML_HTML_DOCUMENT_NODE
-             || self->type == XML_DOCB_DOCUMENT_NODE) {
+    else if (_domIsDoc(self)) {
         // scan document dtds
         xmlDocPtr doc = (xmlDocPtr) self;
         if (doc->intSubset != NULL
@@ -701,8 +703,8 @@ static xmlChar* _domPrepend(xmlChar* str, const char* pfx) {
 // Returns a name that can be used in an XPath filter expression
 DLLEXPORT xmlChar*
 domGetXPathKey(xmlNodePtr node) {
-    xmlChar* temp = NULL;
     xmlChar* rv = NULL;
+
     switch (node->type) {
         case XML_COMMENT_NODE :
             rv = xmlStrdup( (xmlChar*) "comment()");
@@ -732,10 +734,10 @@ domGetXPathKey(xmlNodePtr node) {
                 if  (node->doc && node->doc->type == XML_HTML_DOCUMENT_NODE) {
                     // convert HTML names to lowercase.
                     int i;
-                    char *name = strchr(rv, (int)':');
+                    char *name = strchr((char*)rv, (int)':');
                     if (name == NULL) name = (char*) rv;
 
-                    for (;name[i]; i++) {
+                    for (i = 0; name[i]; i++) {
                         if (name[i] >= 'A' && name[i] <= 'Z') {
                             name[i] += 'a' - 'A';
                         }
@@ -751,7 +753,7 @@ domGetXPathKey(xmlNodePtr node) {
 DLLEXPORT xmlChar*
 domGetASTKey(xmlNodePtr node) {
     xmlChar* rv = NULL;
-    xmlChar* temp = NULL;
+
     switch (node->type) {
         case XML_DOCUMENT_NODE :
             rv = xmlStrdup( (xmlChar*) "#xml");
@@ -903,8 +905,18 @@ domRemoveChild( xmlNodePtr self, xmlNodePtr old ) {
         return NULL;
     }
     if ( self != old->parent ) {
-        /* not a child! */
-        return NULL;
+        int is_parent = 0; // not looking good...
+        if (_domIsDoc(self) && old->type == XML_DTD_NODE) {
+            // ...but there is the special case of doc/dtd relationship
+            xmlDocPtr doc = (xmlDocPtr) self;
+            xmlDtdPtr dtd = (xmlDtdPtr) old;
+            is_parent |= domGetInternalSubset(doc) == dtd;
+            is_parent |= domGetExternalSubset(doc) == dtd;
+        }
+        if (is_parent == 0) {
+            /* old is not a child */
+            return NULL;
+        }
     }
 
     xmlUnlinkNode( old );
@@ -989,11 +1001,8 @@ domInsertBefore( xmlNodePtr self,
     }
 
     if ( refChild != NULL ) {
-        if ( refChild->parent != self
-             || (  newChild->type == XML_DOCUMENT_FRAG_NODE
-                   && newChild->children == NULL ) ) {
-            xmlGenericError(xmlGenericErrorContext,"NOT_FOUND_ERR\n");
-            return NULL;
+        if ( refChild->parent != self ) {
+            xml6_fail(self, "insertBefore/insertAfter: NOT_FOUND_ERR");
         }
     }
 
@@ -1054,7 +1063,7 @@ domReplaceNode( xmlNodePtr self, xmlNodePtr newNode ) {
 
     if ( self->type == XML_ATTRIBUTE_NODE
          || newNode->type == XML_ATTRIBUTE_NODE
-         || newNode->type == XML_DOCUMENT_NODE
+         || _domIsDoc(newNode)
          || domIsParent( newNode, self ) ) {
         /* HIERARCHY_REQUEST_ERR
          * wrong node type
